@@ -1,21 +1,24 @@
 package Persist::Driver::DBI::PostgreSQL;
 
-use 5.8.0;
+use 5.008;
 use strict;
 use warnings;
 
 use DBI;
 
+use DateTime;
+use DateTime::Format::Pg;
+
 use Persist qw(:constants :driver_help);
 use Persist::Driver::DBI;
 use Persist::Filter;
 
 our @ISA = qw(Persist::Driver::DBI);
 
-our ( $VERSION ) = '$Revision: 1.13 $' =~ /\$Revision:\s+([^\s]+)/;
+our ( $VERSION ) = '$Revision: 1.18 $' =~ /\$Revision:\s+([^\s]+)/;
 
 use constant TYPES => [ 
-	qw( varchar int4 serial bool float8 timestamptz )
+	qw( varchar int4 serial bool float8 timestamp )
 ];
 
 =head1 NAME
@@ -67,25 +70,88 @@ sub _parse_conn {
 	\%parsed;
 }
 
-=item $driver = new Persist::Driver::DBI::PostgreSQL($conn, $user, $pass)
+=item $driver = new Persist::Driver::DBI::PostgreSQL(%args)
 
-Creates a new L<DBI> connection using the given connection string C<$conn>,
-username C<$user>, and password C<$pass>.
+Creates a new L<DBI> connection.  The connection is created with AutoCommit set,
+PrintError unset, and RaiseError set.
 
-The connection is created with AutoCommit set, PrintError unset, and RaiseError
-set.
+These arguments C<%args> are accepted: 
+
+=over
+
+=item $uri (optional)
+
+The connection string to connect to PostgreSQL with. If not set, it will be
+created using other arguments.
+
+=item $database (optional)
+
+The name of the database to connect to. This option and C<$uri> are mutually
+exclusive.
+
+=item $host (optional)
+
+The host name to connect to. This option and C<$uri> are mutually exclusive.
+
+=item $port (optional)
+
+The port to connect to. This option and C<$uri> are mutually exclusive.
+
+=item $tty (optional)
+
+The tty option to pass to the PostgreSQL driver. This option and C<$uri> are
+mutually exclusive.
+
+=item $options (optional)
+
+Other options to pass to the PostgreSQL driver. This option and C<$uri> are
+mutually exclusive.
+
+=item $username
+
+The username to use when connecting to the database. If neither C<$uri> or
+C<$database> are given, then this will also be used as the database name.
+
+=item $password (optional)
+
+The password to connect with.
+
+=back
 
 =cut
 
 sub new {
-	my ($class, $conn, $user, $pass) = @_;
+	my ($class, %args) = @_;
+	
+	my ($conn, $user, $pass, $database, $host, $port, $tty, $options) = 
+		@args{qw(-uri -username -password -database -host -port -tty -options)};
 
-	croak "No connection string." unless defined $conn;
+	if (defined $conn and (defined $database or
+						   defined $host or
+						   defined $port or
+						   defined $tty or
+						   defined $options)) {
+		croak "The -uri option cannot be given with any of -database, -host, -port, -tty, or -options.";
+	}
 
-	my $parsed_conn = _parse_conn($conn);
-	croak "Illegal connection string $conn." unless $parsed_conn;
+	croak "No username given for connection." unless defined $user;
 
-	my $self = $class->SUPER::new(DBI->connect
+	my $parsed_conn;
+	if (defined $conn) {
+		$parsed_conn = _parse_conn($conn);
+		croak "Illegal connection string $conn." unless $parsed_conn;
+	} else {
+		$database ||= $user;
+		$parsed_conn->{dbname} = $database;
+		$conn = "dbname=$database";
+		$conn .= ":host=$host" and $parsed_conn->{host} = $host if defined $host;
+		$conn .= ":port=$port" and $parsed_conn->{port} = $port if defined $port;
+		$conn .= ":tty=$tty" and $parsed_conn->{tty} = $tty if defined $tty;
+		$conn .= ":options=$options" and $parsed_conn->{$options} = $options if defined $options;
+	}
+
+	my $self = $class->SUPER::new(
+		-database => DBI->connect
 			("dbi:Pg:$conn", $user, $pass, {
 					AutoCommit => 1,
 					PrintError => 0,
@@ -98,17 +164,17 @@ sub new {
 	$self;
 }
 
-=item $driver->preprocess_filter($tables, $filter)
+=item $driver->preprocess_filter(%args)
 
-This method changes boolean values to literal strings 'true' and 'false' and
-also converts L<Persist> style timestamps to PostgreSQL style dates.
+This method changes boolean values to literal strings 'true' and 'false'.
 
 =cut
 
 sub preprocess_filter {
-	my ($self, $tables, $filter) = @_;
+	my ($self, %args) = @_;
+	my ($tables, $filter) = @args{qw(-aliases -filter)};
 
-	my ($preprocess_boolean, $preprocess_timestamp);
+	my ($preprocess_boolean);
 	my $process_ast = sub { 
 		my ($a, $o, $b) = @{$_[0]};
 		if ($a->isa('Persist::Filter::Identifier') and
@@ -117,12 +183,6 @@ sub preprocess_filter {
 		} elsif ($a->isa('Persist::Filter::Number') and
 				$b->isa('Persist::Filter::Identifier')) {
 			&$preprocess_boolean($b, $a);
-		} elsif ($a->isa('Persist::Filter::Identifier') and
-				$b->isa('Persist::Filter::String')) {
-			&$preprocess_timestamp($a, $b);
-		} elsif ($a->isa('Persist::Filter::String') and
-				$b->isa('Persist::Filter::Identifier')) {
-			&$preprocess_timestamp($b, $a);
 		}
 	};
 	
@@ -130,7 +190,7 @@ sub preprocess_filter {
 	if (scalar(keys %$tables) == 1) {
 		my ($alias) = keys(%$tables);
 		my ($name) = values(%$tables);
-		my $cols = { $self->columns($name) };
+		my $cols = { $self->columns(-table => $name) };
 		
 		$preprocess_boolean = sub {
 			my ($id, $lit) = @_;
@@ -142,24 +202,11 @@ sub preprocess_filter {
 			}
 		};
 
-		$preprocess_timestamp = sub {
-			my ($id, $lit) = @_;
-			for my $col (keys %$cols) {
-				if ($$cols{$col}[0] == TIMESTAMP &&
-						$$id =~ /(?:$alias.)?$col$/i && defined $$lit) {
-					my @ts = Persist->parse_timestamp(substr $$lit, 1, -1);
-					$$lit = sprintf '\'%s%s-%s-%s %s:%s:%s%s %s\'',
-									@ts[1 .. 2], $ts[3] + 1, @ts[4 .. 8],
-									(int($ts[0]) ? 'AD' : 'BC');
-				}
-			}
-		};
-		
 		$ast->remap_on('Persist::Filter::Comparison', $process_ast);
 	} else {
 		my $cols = {};
 		while (my ($alias, $name) = each %$tables) { 
-			$$cols{$alias} = +{ $self->columns($name) };
+			$$cols{$alias} = +{ $self->columns(-table => $name) };
 		}
 
 		$preprocess_boolean = sub {
@@ -174,28 +221,13 @@ sub preprocess_filter {
 			}
 		};
 
-		$preprocess_timestamp = sub {
-			my ($id, $lit) = @_;
-			for my $alias (keys %$cols) {
-				for my $col (keys %{$$cols{$alias}}) {
-					if ($$cols{$alias}{$col}[0] == TIMESTAMP &&
-							$$id =~ /(?:$alias.)?$col$/i && defined $$lit) {
-						my @ts = Persist->parse_timestamp(substr $$lit, 1, -1);
-						$$lit = sprintf '\'%s%s-%s-%s %s:%s:%s%s %s\'',
-									@ts[1 .. 2], $ts[3] + 1, @ts[4 .. 8],
-									(int($ts[0]) ? 'AD' : 'BC');
-					}
-				}
-			}
-		};
-		
 		$ast->remap_on('Persist::Filter::Comparison', $process_ast);
 	}
 
 	$ast->unparse;
 }
 
-=item $rows = $driver-E<gt>insert($name, \%values)
+=item $rows = $driver-E<gt>insert(%args)
 
 This is a wrapper for the C<insert> method of L<Persist::Driver::DBI> which
 converts L<Persist> style dates to PostgreSQL style and boolean values to
@@ -204,24 +236,23 @@ literal strings 'true' and 'false'.
 =cut
 
 sub insert {
-	my ($self, $name, $values) = @_;
+	my ($self, %args) = @_;
+	my ($name, $values) = @args{qw(-table -values)};
 
-	my %columns = $self->columns($name);
+	my %columns = $self->columns(-table => $name);
 	while (my ($k, $v) = each %columns) {
-		if ($v->[0] == TIMESTAMP and defined($values->{$k})) {
-			my @ts = Persist->parse_timestamp($values->{$k});
-			$values->{$k} = sprintf '%s%s-%s-%s %s:%s:%s%s %s',
-					@ts[1 .. 2], $ts[3] + 1, @ts[4 .. 8],
-					(int($ts[0]) ? 'AD' : 'BC');
+		if ($v->[0] == TIMESTAMP and defined($values->{$k})
+								 and ref $values->{$k} eq 'DateTime') {
+			$values->{$k} = DateTime::Format::Pg->format_timestamp($values->{$k});
 		} elsif ($v->[0] == BOOLEAN and defined($values->{$k})) {
 			$values->{$k} = ($values->{$k} ? 'true' : 'false');
 		}
 	}
 
-	$self->SUPER::insert($name, $values);
+	$self->SUPER::insert(-table => $name, -values => $values);
 }
 
-=item $rows = $driver-E<gt>update($name, \%set [, $filter [, \@bindings ] ] )
+=item $rows = $driver-E<gt>update(%args)
 
 This is a wrapper for the C<update> method of L<Persist::Driver::DBI> which
 converts L<Persist> style dates to PostgreSQL style and boolean values to
@@ -230,24 +261,23 @@ literal strings 'true' and 'false'.
 =cut
 
 sub update {
-	my ($self, $name, $set, $filter, $bindings) = @_;
+	my ($self, %args) = @_;
+	my ($name, $set, $filter, $bindings) = @args{qw(-table -set -filter -bindings)};
 
-	my %columns = $self->columns($name);
+	my %columns = $self->columns(-table => $name);
 	while (my ($k, $v) = each %columns) {
-		if ($v->[0] == TIMESTAMP and defined($set->{$k})) {
-			my @ts = Persist->parse_timestamp($set->{$k});
-			$set->{$k} = sprintf '%s%s-%s-%s %s:%s:%s%s %s',
-					@ts[1 .. 2], $ts[3] + 1, @ts[4 .. 8],
-					($ts[0] eq '+' ? 'AD' : 'BC');
+		if ($v->[0] == TIMESTAMP and defined($set->{$k})
+								 and ref $set->{$k} eq 'DateTime') {
+			$set->{$k} = DateTime::Format::Pg->format_timestamp($set->{$k});
 		} elsif ($v->[0] == BOOLEAN and defined($set->{$k})) {
 			$set->{$k} = ($set->{$k} ? 'true' : 'false');
 		}
 	}
 
-	$self->SUPER::update($name, $set, $filter, $bindings);
+	$self->SUPER::update(-table => $name, -set => $set, -filter => $filter, -bindings => $bindings);
 }
 
-=item $driver->first($handle)
+=item $driver->first(%args)
 
 This is a wrapper for the C<first> method of L<Persist::Driver::DBI> and
 converts PostgreSQL style dates to L<Persist> style timestamps.
@@ -255,22 +285,18 @@ converts PostgreSQL style dates to L<Persist> style timestamps.
 =cut
 
 sub first {
-	my ($self, $handle) = @_;
+	my ($self, %args) = @_;
+	my $handle = $args{-handle};
 	my $tables = $handle->[0];
 
-	my $results = $self->SUPER::first($handle);
+	my $results = $self->SUPER::first(-handle => $handle);
 
 	if (defined $results) {
 		for my $name (@$tables) {
-			my %columns = $self->columns($name);
+			my %columns = $self->columns(-table => $name);
 			while (my ($k, $v) = each %columns) {
 				if ($v->[0] == TIMESTAMP and defined($results->{$k})) {
-					my @ts = $results->{$k} =~ /(\d+)-(\d+)-(\d+) (\d+):(\d+):(\d+)((?:\+|-)\d*)? ?((?:AD|BC)?)/;
-					$results->{$k} = join '',
-							(defined $ts[7] and $ts[7] eq 'BC' ? '-' : '+'),
-							$ts[0], (sprintf '%02d', $ts[1] - 1), @ts[2 .. 5],
-							(defined $ts[6] and length($ts[6]) == 0 ? '+0000' :
-								(defined $ts[6] and length($ts[6]) == 3 ? $ts[6].'00' : $ts[6]));
+					$results->{$k} = DateTime::Format::Pg->parse_timestamp($results->{$k});
 				}
 			}
 		}
@@ -279,32 +305,28 @@ sub first {
 	$results;
 }
 
-=item $driver->next($handle)
+=item $driver->next(%args)
 
 This is a wrapper for the C<first> method of L<Persist::Driver::DBI> and
-converts PostgreSQL style dates to L<Persist> style timestamps.
+converts PostgreSQL dates to L<DateTime> objects.
 
 =cut
 
 # TODO Combine common code in first and next.
 
 sub next {
-	my ($self, $handle) = @_;
+	my ($self, %args) = @_;
+	my $handle = $args{-handle};
 	my $tables = $handle->[0];
 
-	my $results = $self->SUPER::next($handle);
+	my $results = $self->SUPER::next(-handle => $handle);
 
 	if (defined $results) {
 		for my $name (@$tables) {
-			my %columns = $self->columns($name);
+			my %columns = $self->columns(-table => $name);
 			while (my ($k, $v) = each %columns) {
 				if ($v->[0] == TIMESTAMP and defined($results->{$k})) {
-					my @ts = $results->{$k} =~ /(\d+)-(\d+)-(\d+) (\d+):(\d+):(\d+)((?:\+|-)\d*)? ?((?:AD|BC)?)/;
-					$results->{$k} = join '',
-							(defined $ts[7] and $ts[7] eq 'BC' ? '-' : '+'),
-							$ts[0], (sprintf '%02d', $ts[1] - 1), @ts[2 .. 5],
-							(defined $ts[6] and length($ts[6]) == 0 ? '+0000' :
-								(defined $ts[6] and length($ts[6]) == 3 ? $ts[6].'00' : $ts[6]));
+					$results->{$k} = DateTime::Format::Pg->parse_timestamp($results->{$k});
 				}
 			}
 		}
@@ -341,17 +363,32 @@ sub is_dba {
 	@$users == 1;
 }
 
-=item @args = $driver->new_source($user, $pass)
+=item @args = $driver->new_source(%args)
 
-Creates a new data source using the given username C<$user> and password
-C<$pass> and returns the connection string to connect to the new database. This
-will create a new PostgreSQL user with the given name and then a database of
-the same name. The user will not be permitted to create databases or users.
+Creates a new data source and returns the arguments required to connect to the
+new database. This will create a new PostgreSQL user with the given name and
+then a database of the same name. The user will not be permitted to create
+databases or users.
+
+The arguments C<%args> accepted are:
+
+=over
+
+=item $username
+
+The name of the username and database to create.
+
+=item $password
+
+The password used to access this database.
+
+=back
 
 =cut
 
 sub new_source {
-	my ($self, $user, $pass) = @_;
+	my ($self, %args) = @_;
+	my ($user, $pass) = @args{qw(-username -password)};
 
 	$self->handle->do(qq(
 		CREATE USER $user PASSWORD '$pass' NOCREATEDB NOCREATEUSER
@@ -368,17 +405,28 @@ sub new_source {
 		push @opts, "$key=$value";
 	}
 	
-	(join(';', @opts), $user, $pass);
+	(-uri => join(';', @opts), -username => $user, -password => $pass);
 }
 
-=item $success = $driver->delete_source($user)
+=item $success = $driver->delete_source(%args)
 
-Deletes the database and user associated with the username C<$user>.
+Deletes the database and user given.
+
+The arguments C<%args> accepted are:
+
+=over
+
+=item $username
+
+The name of the user and database to be dropped.
+
+=back
 
 =cut
 
 sub delete_source {
-	my ($self, $user) = @_;
+	my ($self, %args) = @_;
+	my $user = $args{-username};
 
 	croak "No source specified for deletion." unless defined $user;
 
@@ -393,7 +441,7 @@ sub delete_source {
 	1;
 }
 
-=item $success = $driver->create_table($name, $columns, $indexes)
+=item $success = $driver->create_table(%args)
 
 Creates the table as described by the arguments. All C<PRIMARY>, C<UNIQUE>, and
 C<LINK> keys will be added as constraints. See L</INDEXES> for details on how
@@ -402,7 +450,11 @@ these used. See L</TYPES> for information on column types.
 =cut
 
 sub create_table {
-    my ($self, $name, $columns, $indexes) = @_;
+    my ($self, %args) = @_;
+	my ($name, $columns, $indexes) = @args{qw(-table -columns -indexes)};
+	
+	# For now, we ignore column order (YUCK!)
+	$columns = { @$columns };
 
     my $sql = "CREATE TABLE $name(";
 
@@ -449,14 +501,15 @@ sub create_table {
 	1;
 }
 
-=item $success = $driver->delete_table($name)
+=item $success = $driver->delete_table(%args)
 
 Deletes the given table from the database.
 
 =cut
 
 sub delete_table {
-	my ($self, $name) = @_;
+	my ($self, %args) = @_;
+	my $name = $args{-table};
 
 	eval {
 		$self->handle->do(qq(
@@ -469,7 +522,7 @@ sub delete_table {
 	1;
 }
 
-=item %columns = $driver->columns($name)
+=item %columns = $driver->columns(%args)
 
 Recreates the column definition used to create the table. This should work even
 if the Persist driver didn't create the table, as long as it understands all
@@ -481,7 +534,8 @@ See L</TYPES> for details on what types are used.
 =cut
 
 sub columns {
-	my ($self, $name) = @_;
+	my ($self, %args) = @_;
+	my $name = $args{-table};
 
 #	print STDERR "name: $name\n";
 #	print STDERR "caller: ",join(',', caller),"\n";
@@ -525,12 +579,12 @@ sub columns {
 				@type = ( REAL );
 				last SWITCH;
 			};
-			/^timestamptz$/i && do {
+			/^timestamp$/i && do {
 				@type = ( TIMESTAMP );
 				last SWITCH;
 			};
 			DEFAULT: do {
-				croak "Unknown type \"$_\".";
+				croak "Found $name.$column_name with unknown type \"$_\".";
 			};
 		}
 		$results{$column_name} = [ @type ];
@@ -539,7 +593,7 @@ sub columns {
 	%results;
 }
 
-=item @indexes = $driver->indexes($name)
+=item @indexes = $driver->indexes(%args)
 
 Recreates the index definition used to create the table. This should work even
 if the Persist driver didn't create the table. This information is created by
@@ -550,7 +604,8 @@ See L</INDEXES> for information on how indexes are mapped.
 =cut
 
 sub indexes {
-	my ($self, $name) = @_;
+	my ($self, %args) = @_;
+	my $name = $args{-table};
 
 	my $sth = $self->handle->primary_key_info(undef, undef, $name);
 	my $rows = $sth->fetchall_arrayref({});
@@ -618,7 +673,7 @@ sub indexes {
 	@results;
 }
 
-=item $value = $driver-E<gt>sequence_value($table, $column)
+=item $value = $driver-E<gt>sequence_value(%args)
 
 Returns the last C<AUTONUMBER> value used during an insert. An insert must have
 been performed since the database connection was created for this method to
@@ -627,7 +682,8 @@ succeed.
 =cut
 
 sub sequence_value {
-	my ($self, $table, $column) = @_;
+	my ($self, %args) = @_;
+	my ($table, $column) = @args{qw(-table -column)};
 
 	$self->handle->selectcol_arrayref(qq(
 		SELECT CURRVAL('${table}_${column}_seq')
@@ -659,7 +715,7 @@ At this time, L<Persist> types are mapped as this table indicates:
   AUTONUMBER                       serial
   BOOLEAN                          bool
   REAL                             float8
-  TIMESTAMP                        timestamptz
+  TIMESTAMP                        timestamp
 
 The opposite direction isn't quite as easy as C<serial> is a pseudonym for
 C<int4> with an attached sequence. When mapping in the opposite direction, any
